import _extends from "@babel/runtime/helpers/extends";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
import React from 'react';
import PropTypes from 'prop-types';
import Input from "semantic-ui-react/dist/es/elements/Input/Input.js";
import { validationError, validationWarning } from '../helpers/helpers';
import useFieldApi from "@data-driven-forms/react-form-renderer/use-field-api";
import FormFieldGrid from '../form-field-grid/form-field-grid';
import FormField from '../form-field/form-field';

var TextField = function TextField(props) {
  var _useFieldApi = useFieldApi(props),
      input = _useFieldApi.input,
      isReadOnly = _useFieldApi.isReadOnly,
      isDisabled = _useFieldApi.isDisabled,
      placeholder = _useFieldApi.placeholder,
      isRequired = _useFieldApi.isRequired,
      label = _useFieldApi.label,
      helperText = _useFieldApi.helperText,
      validateOnMount = _useFieldApi.validateOnMount,
      meta = _useFieldApi.meta,
      FormFieldGridProps = _useFieldApi.FormFieldGridProps,
      HelpertextProps = _useFieldApi.HelpertextProps,
      rest = _objectWithoutProperties(_useFieldApi, ["input", "isReadOnly", "isDisabled", "placeholder", "isRequired", "label", "helperText", "validateOnMount", "meta", "FormFieldGridProps", "HelpertextProps"]);

  var invalid = validationError(meta, validateOnMount);
  return /*#__PURE__*/React.createElement(FormFieldGrid, _extends({
    helperText: validationWarning(meta, validateOnMount) || helperText,
    HelpertextProps: HelpertextProps
  }, FormFieldGridProps), /*#__PURE__*/React.createElement(FormField, _extends({}, rest, {
    control: Input,
    label: label,
    required: isRequired,
    disabled: isDisabled,
    readOnly: isReadOnly,
    placeholder: placeholder
  }, input, {
    error: invalid && {
      content: meta.error || meta.submitError
    }
  })));
};

TextField.propTypes = {
  isReadOnly: PropTypes.bool,
  isDisabled: PropTypes.bool,
  placeholder: PropTypes.node,
  isRequired: PropTypes.bool,
  label: PropTypes.node,
  helperText: PropTypes.node,
  validateOnMount: PropTypes.bool,

  /** Sub component customization */
  FormFieldGridProps: PropTypes.object,
  HelpertextProps: PropTypes.object
};
TextField.defaultProps = {
  FormFieldGridProps: {},
  HelpertextProps: {}
};
export default TextField;